import filecmp
import pathlib
from os import path

import pytest

from delivery.deliverer import LocalDestination


@pytest.fixture
def dest_dir(tmpdir) -> pathlib.Path:
    """
    Generate a destination directory.
    :param tmpdir:  the tmpdir fixture we depend on
    :return:  path to the destination directory
    """
    # generate the destination directory
    dest_dir = pathlib.Path(tmpdir) / "after"
    dest_dir.mkdir()
    return dest_dir


@pytest.fixture
def file_to_deliver(tmpdir) -> pathlib.Path:
    """
    Generate some files in a test directory for delivery.
    :param tmpdir: pytest fixture for the temporary directory
    :return: a file in the test directory
    """
    # rewrap the tmpdir as a pathlib.Path instance
    tmpdir = pathlib.Path(tmpdir)

    # generate the test file in prior
    (tmpdir / "prior").mkdir()
    file_name = "test.txt"

    temp_file = tmpdir / "prior" / file_name  # place a file into the source
    temp_file.write_text("content", encoding=None)  # write something into that file

    # secretly generate a second file
    temp_file2 = tmpdir / "prior" / "test2.txt"  # make another file in source
    temp_file2.write_text("more content", encoding=None)  # add some content to the second file

    return temp_file


def test_local_delivery_add_file(tmpdir, file_to_deliver: pathlib.Path, dest_dir: pathlib.Path):
    """
    Ensure that local delivery works
    """
    # generate the destination
    local_dest = LocalDestination(None, dest_dir)

    # add the file to it
    local_dest.add_file(file_to_deliver, file_to_deliver.name)

    # see if the source file eneded up in the destination
    assert (dest_dir / file_to_deliver.name).exists()

    # see if the content of the file is intact
    assert (dest_dir / file_to_deliver.name).read_text(encoding=None) == "content"

    # see if the results are alright
    results = local_dest.results()
    assert len(results.keys()) == 1
    assert results["delivered_to"] == str(dest_dir)


def test_local_delivery_add_directory(
    tmpdir: pathlib.Path, file_to_deliver: pathlib.Path, dest_dir: pathlib.Path
):
    """Ensure that local delivery does something"""
    tmpdir = pathlib.Path(tmpdir)

    local_dest = LocalDestination(None, dest_dir)
    local_dest.add_directory(file_to_deliver.parent, file_to_deliver.parent.name)

    compare_dirs = filecmp.dircmp(file_to_deliver.parent, dest_dir / file_to_deliver.parent)

    # see if the destination got all the files from source
    assert len(compare_dirs.left_only) == 0
    assert len(compare_dirs.right_only) == 0
    assert len(compare_dirs.funny_files) == 0

    # see if the results are alright
    results = local_dest.results()
    assert len(results.keys()) == 1
    assert results["delivered_to"] == str(dest_dir)


def test_subdirectories(
    tmpdir: pathlib.Path, file_to_deliver: pathlib.Path, dest_dir: pathlib.Path
):
    """
    Ensure that subdirectories work the way we expect them to.
    """
    subdirectory = "subdirectory"

    tmpdir = pathlib.Path(tmpdir)
    local_dest = LocalDestination(None, dest_dir)
    subdir = local_dest / subdirectory
    subdir.add_directory(file_to_deliver.parent, file_to_deliver.parent.name)

    # ensure that we only got one file in the tmpdir called "subdirectory"
    assert list(dest_dir.iterdir()) == [dest_dir / subdirectory]

    # compare the subdirectories
    compare_dirs = filecmp.dircmp(
        file_to_deliver.parent, dest_dir / subdirectory / file_to_deliver.parent
    )

    # ensure that the destination got all the files in the source
    assert len(compare_dirs.left_only) == 0
    assert len(compare_dirs.right_only) == 0
    assert len(compare_dirs.funny_files) == 0
